/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sga;

import java.io.FileInputStream;
import java.util.Properties;

/**
 * Class POEMSProperties is an internal representation of the configuration 
 * parameters of the algorithm poems. All parameters are read from the file 
 * PoemsConf.properties.
 *
 * @author Michal Snizek
 */
public class SGAParameters extends Properties {

    private int maxGenes = 24;         // chromosome size
    private double minX = 0.0;
    private double maxX = 150.0;    
    private int popSize = 100;         // population size
    private int evaluations = 10000;    // number of fitness evaluations
    private int crossType = 1;         // crossover type
    private double pC = 0.75;             // crossover probability
    private double pM = 0.01;             // mutation probability
    private int selectionType = 0;     // selection type
    private int tournamentSize = 3;    // tournament size parameter
    //---
    private static SGAParameters instance;
    
    /**
     * Private constructor.
     */
    private SGAParameters() {
    }

    /**
     * Class POEMS Properties is an implementation of singleton design pattern. 
     * This method returns the instance of this class.
     * 
     * @return instance of class POEMSProperties
     */
    public static SGAParameters getInstance() {
        if (instance == null) {
            instance = new SGAParameters();
        }
        return instance;
    }

    /**
     * This method will read the configuration file and set the values for the
     * POEMS run.
     */
    public void readConfigFile() throws Exception {

        FileInputStream fis = new FileInputStream("SGAConfiguration.properties");
        this.load(fis);
      
        try{
            popSize = Integer.parseInt(this.getProperty("popSize"));
        }
        catch(Exception e){
            System.out.println("Uses default popSize");   
        }
        try{
            maxGenes = Integer.parseInt(this.getProperty("maxGenes"));
        }
        catch(Exception e){
            System.out.println("Uses default maxGenes");   
        }
        try{
            minX = Double.parseDouble(this.getProperty("minX"));
        }
        catch(Exception e){
            System.out.println("Uses default minX");   
        }
        try{
            maxX = Double.parseDouble(this.getProperty("maxX"));
        }
        catch(Exception e){
            System.out.println("Uses default maxX");   
        }
        try{
            evaluations = Integer.parseInt(this.getProperty("evaluations"));
        }
        catch(Exception e){
            System.out.println("Uses default evaluations");   
        }
        try{
            crossType = Integer.parseInt(this.getProperty("crossType"));
        }
        catch(Exception e){
            System.out.println("Uses default crossType");   
        }
        try{
            pC = Double.parseDouble(this.getProperty("pC"));
        }
        catch(Exception e){
            System.out.println("Uses default pC");   
        }
        try{
            pM = Double.parseDouble(this.getProperty("pM"));
        }
        catch(Exception e){
            System.out.println("Uses default pM");   
        }
        try{
            selectionType = Integer.parseInt(this.getProperty("selectionType"));
        }
        catch(Exception e){
            System.out.println("Uses default selectionType");   
        }
        try{
            tournamentSize = Integer.parseInt(this.getProperty("tournamentSize"));
        }
        catch(Exception e){
            System.out.println("Uses default tournamentSize");   
        }

        if (popSize <= 4) {
            throw new Exception("check configuration: popSize must be greater than 4");
        }
        if (maxGenes < 4) {
            throw new Exception("check configuration: maxGenes must be greater than 4");
        }
        if (minX >= maxX) {
            throw new Exception("check configuration: minX must be smaller than maxX");
        }
        if (evaluations < popSize) {
            throw new Exception("check configuration: evaluations must be greater or equal than popSize");
        }
        if ((crossType < 0) || (crossType > 2)) {
            throw new Exception("check configuration: crossType must be either 0 or 1 or 2");
        }
        if ((pC < 0.0) || (pC > 1.0)) {
            throw new Exception("check configuration: crossover probability must be within (0.0, 1.0)");
        }
        if ((pM < 0.0) || (pM > 1.0)) {
            throw new Exception("check configuration: mutation probability must be within (0.0, 1.0)");
        }
        if (tournamentSize <= 0) {
            throw new Exception("check configuration: tournamentSize must be greater than 0");
        }
    }

    public int getPopSize() {
        return popSize;
    }
    
    public int getMaxGenes() {
        return maxGenes;
    }

    public double getMinX() {
        return minX;
    }

    public double getMaxX() {
        return maxX;
    }

    public int getEvaluations() {
        return evaluations;
    }
    
    public int getCrossType() {
        return crossType;
    }

    public double getPC() {
        return pC;
    }

    public double getPM() {
        return pM;
    }

    public int getSelectionType() {
        return selectionType;
    }

    public int getTournamentSize() {
        return tournamentSize;
    }
}
